import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _createSuper from "@babel/runtime/helpers/esm/createSuper";
var _excluded = ["type"];
import Component from '../base/component';
import equal from '../base/equal';
import { applyMixins } from '../mixins';
import InteractionMixin from '../mixins/interaction';
import Children from '../children'; // types

import LayoutController from '../controller/layout';
import CoordController from '../controller/coord';
import ScaleController from '../controller/scale'; // 统计图表

var Chart = /*#__PURE__*/function (_Component) {
  _inherits(Chart, _Component);

  var _super = _createSuper(Chart);

  function Chart(props, context, updater) {
    var _this;

    _classCallCheck(this, Chart);

    _this = _super.call(this, props, context, updater);
    var data = props.data,
        coordOption = props.coord,
        scale = props.scale,
        _props$interactions = props.interactions,
        interactions = _props$interactions === void 0 ? [] : _props$interactions;
    _this.layoutController = new LayoutController();
    _this.coordController = new CoordController();
    _this.scaleController = new ScaleController(data);
    _this.scale = _this.scaleController;

    var _assertThisInitialize = _assertThisInitialized(_this),
        layoutController = _assertThisInitialize.layoutController,
        coordController = _assertThisInitialize.coordController,
        scaleController = _assertThisInitialize.scaleController; // 布局


    var style = _this.getStyle(props, context);

    _this.layout = layoutController.create(style); // 坐标系

    _this.coord = coordController.create(coordOption, _this.layout); // scale

    scaleController.create(scale); // 创建交互事件控制器

    var interactionController = _this.createInteractionController({
      chart: _assertThisInitialized(_this)
    }); // 定义事件


    interactions.forEach(function (interaction) {
      var type = interaction.type,
          cfg = _objectWithoutProperties(interaction, _excluded);

      interactionController.createInteraction(type, cfg);
    });
    _this.data = data;
    _this.interaction = interactionController; // state

    _this.state = {
      zoomRange: [0, 1]
    };
    return _this;
  } // props 更新


  _createClass(Chart, [{
    key: "willReceiveProps",
    value: function willReceiveProps(nextProps) {
      var layoutController = this.layoutController,
          coordController = this.coordController,
          scaleController = this.scaleController,
          lastProps = this.props;
      var nextStyle = nextProps.style,
          nextData = nextProps.data,
          nextScale = nextProps.scale,
          nextInteractions = nextProps.interactions;
      var lastStyle = lastProps.style,
          lastData = lastProps.data,
          lastScale = lastProps.scale,
          lastInteractions = lastProps.interactions; // 布局

      if (!equal(nextStyle, lastStyle)) {
        var style = this.getStyle(nextProps, this.context);
        this.layout = layoutController.create(style);
        coordController.updateLayout(this.layout);
      }

      if (nextData !== lastData) {
        scaleController.changeData(nextData);
      } // scale


      if (!equal(nextScale, lastScale)) {
        scaleController.update(nextScale);
      }
    }
  }, {
    key: "willUpdate",
    value: function willUpdate() {
      var coordController = this.coordController,
          props = this.props; // render 时要重置 coord 范围，重置后需要让所有子组件都重新render
      // 所以这里不比较是否有差异，每次都新建，让所有子组件重新render

      this.coord = coordController.create(props.coord, this.layout);
    }
  }, {
    key: "getStyle",
    value: function getStyle(props, context) {
      var theme = context.theme,
          px2hd = context.px2hd,
          left = context.left,
          top = context.top,
          width = context.width,
          height = context.height;
      var style = props.style;
      return px2hd(_objectSpread(_objectSpread({
        left: left,
        top: top,
        width: width,
        height: height
      }, theme.chart), style));
    }
  }, {
    key: "layoutCoord",
    value: function layoutCoord(position, box) {
      var coord = this.coord;
      var boxWidth = box.width,
          boxHeight = box.height;
      var left = coord.left,
          top = coord.top,
          width = coord.width,
          height = coord.height;

      switch (position) {
        case 'left':
          left += boxWidth;
          width = Math.max(0, width - boxWidth);
          break;

        case 'right':
          width = Math.max(0, width - boxWidth);
          break;

        case 'top':
          top += boxHeight;
          height = Math.max(0, height - boxHeight);
          break;

        case 'bottom':
          height = Math.max(0, height - boxHeight);
          break;
      }

      coord.update({
        left: left,
        top: top,
        width: width,
        height: height
      });
    }
  }, {
    key: "getGeometrys",
    value: function getGeometrys() {
      var children = this.children;
      var geometrys = [];
      Children.toArray(children).forEach(function (element) {
        if (!element) return false;
        var component = element.component;

        if (component && component.isGeometry) {
          geometrys.push(component);
        }
      });
      return geometrys;
    }
  }, {
    key: "getSnapRecords",
    value: function getSnapRecords(point) {
      var geometrys = this.getGeometrys();
      if (!geometrys.length) return; // @ts-ignore

      return geometrys[0].getSnapRecords(point);
    }
  }, {
    key: "getLegendItems",
    value: function getLegendItems(point) {
      var geometrys = this.getGeometrys();
      if (!geometrys.length) return; // @ts-ignore

      return geometrys[0].getLegendItems(point);
    }
  }, {
    key: "setScale",
    value: function setScale(field, option) {
      this.scaleController.setScale(field, option);
    }
  }, {
    key: "getScale",
    value: function getScale(field) {
      return this.scaleController.getScale(field);
    }
  }, {
    key: "getScales",
    value: function getScales() {
      return this.scaleController.getScales();
    }
  }, {
    key: "getXScales",
    value: function getXScales() {
      var geometrys = this.getGeometrys();
      return geometrys.map(function (component) {
        // @ts-ignore
        return component.getXScale();
      });
    }
  }, {
    key: "getYScales",
    value: function getYScales() {
      var geometrys = this.getGeometrys();
      return geometrys.map(function (component) {
        // @ts-ignore
        return component.getYScale();
      });
    }
  }, {
    key: "getCoord",
    value: function getCoord() {
      return this.coord;
    }
  }, {
    key: "render",
    value: function render() {
      var _this2 = this;

      var props = this.props,
          state = this.state,
          layout = this.layout,
          coord = this.coord;
      var children = props.children,
          data = props.data;
      return Children.map(children, function (child) {
        return Children.cloneElement(child, {
          chart: _this2,
          data: data,
          coord: coord,
          layout: layout,
          zoomRange: state.zoomRange
        });
      });
    }
  }]);

  return Chart;
}(Component); // 多继承


applyMixins(Chart, [InteractionMixin]);

var ExportChart = /*#__PURE__*/function (_Chart) {
  _inherits(ExportChart, _Chart);

  var _super2 = _createSuper(ExportChart);

  function ExportChart() {
    _classCallCheck(this, ExportChart);

    return _super2.apply(this, arguments);
  }

  return _createClass(ExportChart);
}(Chart);

export default ExportChart;