import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { isFunction } from '@antv/util';
import { jsx } from '../../jsx';

var Marker = function Marker(_ref) {
  var type = _ref.type,
      color = _ref.color;

  if (type === 'square') {
    return jsx("rect", {
      style: {
        width: '20px',
        height: '20px',
        marginRight: '6px',
        marginTop: '6px'
      },
      attrs: {
        fill: color,
        width: '14px',
        height: '14px'
      }
    });
  }

  return jsx("circle", {
    style: {
      width: '20px',
      height: '20px',
      marginRight: '6px'
    },
    attrs: {
      fill: color,
      r: '6px'
    }
  });
};

export default (function (props) {
  var items = props.items,
      layout = props.layout,
      position = props.position,
      maxItemWidth = props.maxItemWidth,
      itemFormatter = props.itemFormatter,
      style = props.style,
      _props$marker = props.marker,
      marker = _props$marker === void 0 ? 'circle' : _props$marker;
  var left = layout.left,
      top = layout.top,
      right = layout.right,
      bottom = layout.bottom,
      width = layout.width,
      height = layout.height;

  var legendStyle = _objectSpread({
    // default style
    width: width,
    flexDirection: 'row',
    flexWrap: 'wrap',
    padding: [0, '6px', '20px', 0],
    justifyContent: 'center'
  }, style);

  var isVertical = position === 'left' || position === 'right';

  var formatValue = function formatValue(value) {
    if (isFunction(itemFormatter)) {
      return ": ".concat(itemFormatter(value));
    }

    return ": ".concat(value);
  };

  if (isVertical) {
    legendStyle.width = maxItemWidth;
    legendStyle.flexDirection = 'column';
    legendStyle.alignItems = 'flex-start';
  }

  if (position === 'left' || position === 'top') {
    legendStyle.top = top;
    legendStyle.left = left;
  }

  if (position === 'right') {
    legendStyle.top = top;
    legendStyle.left = right - maxItemWidth;
  }

  if (position === 'bottom') {
    legendStyle.top = height + top;
    legendStyle.left = left;
  }

  return jsx("group", {
    style: legendStyle
  }, items.map(function (item) {
    var color = item.color,
        name = item.name,
        value = item.value;
    return jsx("group", {
      style: {
        display: 'flex',
        flexDirection: 'row',
        alignItems: 'center',
        justifyContent: 'center',
        padding: ['6px', 0, 0, '6px']
      }
    }, jsx(Marker, {
      color: color,
      type: marker
    }), jsx("text", {
      attrs: {
        fill: 'black',
        text: name
      }
    }), value ? jsx("text", {
      attrs: {
        fill: 'black',
        text: formatValue(value)
      }
    }) : null);
  }));
});