import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _inherits from "@babel/runtime/helpers/esm/inherits";
import _createSuper from "@babel/runtime/helpers/esm/createSuper";
import { isFunction } from '@antv/util';
import Component from '../../base/component';
import { jsx } from '../../jsx'; // view 的默认配置

var defaultStyle = {
  showTitle: false,
  showCrosshairs: false,
  crosshairsType: 'y',
  crosshairsStyle: {
    stroke: 'rgba(0, 0, 0, 0.25)',
    lineWidth: '2px'
  },
  showTooltipMarker: true,
  tooltipMarkerStyle: {
    fill: '#fff',
    lineWidth: '3px'
  },
  background: {
    radius: '2px',
    fill: 'rgba(0, 0, 0, 0.65)',
    padding: ['6px', '10px']
  },
  titleStyle: {
    fontSize: '24px',
    fill: '#fff',
    textAlign: 'start',
    textBaseline: 'top'
  },
  nameStyle: {
    fontSize: '24px',
    fill: 'rgba(255, 255, 255, 0.65)',
    textAlign: 'start',
    textBaseline: 'middle'
  },
  valueStyle: {
    fontSize: '24px',
    fill: '#fff',
    textAlign: 'start',
    textBaseline: 'middle'
  },
  joinString: ': ',
  showItemMarker: true,
  itemMarkerStyle: {
    width: '12px',
    radius: '6px',
    symbol: 'circle',
    lineWidth: '2px',
    stroke: '#fff'
  },
  layout: 'horizontal',
  snap: false,
  xTipTextStyle: {
    fontSize: 12,
    fill: '#fff'
  },
  yTipTextStyle: {
    fontSize: 12,
    fill: '#fff'
  },
  xTipBackground: {
    radius: '2px',
    fill: 'rgba(0, 0, 0, 0.65)',
    padding: ['6px', '10px'],
    marginLeft: "-50%"
  },
  yTipBackground: {
    radius: '2px',
    fill: 'rgba(0, 0, 0, 0.65)',
    padding: ['6px', '10px'],
    marginLeft: "-100%",
    marginTop: "-50%"
  }
};

function directionEnabled(mode, dir) {
  if (mode === undefined) {
    return true;
  } else if (typeof mode === 'string') {
    return mode.indexOf(dir) !== -1;
  }

  return false;
}

var TooltipView = /*#__PURE__*/function (_Component) {
  _inherits(TooltipView, _Component);

  var _super = _createSuper(TooltipView);

  function TooltipView(props) {
    var _this;

    _classCallCheck(this, TooltipView);

    _this = _super.call(this, props);
    _this.rootRef = {
      current: null
    };
    _this.itemsRef = {
      current: null
    };
    return _this;
  }

  _createClass(TooltipView, [{
    key: "didUpdate",
    value: function didUpdate() {
      var props = this.props,
          rootRef = this.rootRef,
          itemsRef = this.itemsRef;
      var group = rootRef.current;

      if (!group) {
        return;
      }

      var records = props.records,
          coord = props.coord;
      if (!records || !records.length) return null;

      if (itemsRef.current) {
        var firstRecord = records[0];
        var x = firstRecord.x;
        var coordLeft = coord.left,
            coordWidth = coord.width;

        var _group$get = group.get('attrs'),
            width = _group$get.width;

        var halfWidth = width / 2;
        var moveX = Math.min(Math.max(x - coordLeft - halfWidth, 0), coordWidth - halfWidth);
        itemsRef.current.moveTo(moveX, 0);
      }
    }
  }, {
    key: "render",
    value: function render() {
      var props = this.props;
      var records = props.records,
          chart = props.chart,
          coord = props.coord,
          layout = props.layout;
      var layoutTop = layout.top;
      if (!records || !records.length) return null;
      var coordLeft = coord.left,
          coordTop = coord.top,
          coordRight = coord.right,
          coordBottom = coord.bottom;
      var firstRecord = records[0];
      var x = firstRecord.x,
          y = firstRecord.y,
          xField = firstRecord.xField,
          yField = firstRecord.yField,
          firstOrigin = firstRecord.origin;
      var yScale = chart.getScale(yField);
      var xScale = chart.getScale(xField);
      var background = props.background,
          showTitle = props.showTitle,
          titleStyle = props.titleStyle,
          _props$showItemMarker = props.showItemMarker,
          showItemMarker = _props$showItemMarker === void 0 ? defaultStyle.showItemMarker : _props$showItemMarker,
          customItemMarkerStyle = props.itemMarkerStyle,
          nameStyle = props.nameStyle,
          valueStyle = props.valueStyle,
          _props$joinString = props.joinString,
          joinString = _props$joinString === void 0 ? defaultStyle.joinString : _props$joinString,
          _props$showCrosshairs = props.showCrosshairs,
          showCrosshairs = _props$showCrosshairs === void 0 ? defaultStyle.showCrosshairs : _props$showCrosshairs,
          crosshairsStyle = props.crosshairsStyle,
          _props$crosshairsType = props.crosshairsType,
          crosshairsType = _props$crosshairsType === void 0 ? defaultStyle.crosshairsType : _props$crosshairsType,
          _props$snap = props.snap,
          snap = _props$snap === void 0 ? defaultStyle.snap : _props$snap,
          _props$tooltipMarkerS = props.tooltipMarkerStyle,
          tooltipMarkerStyle = _props$tooltipMarkerS === void 0 ? defaultStyle.tooltipMarkerStyle : _props$tooltipMarkerS,
          showXTip = props.showXTip,
          showYTip = props.showYTip,
          xTip = props.xTip,
          yTip = props.yTip,
          _props$xTipTextStyle = props.xTipTextStyle,
          xTipTextStyle = _props$xTipTextStyle === void 0 ? defaultStyle.xTipTextStyle : _props$xTipTextStyle,
          _props$yTipTextStyle = props.yTipTextStyle,
          yTipTextStyle = _props$yTipTextStyle === void 0 ? defaultStyle.yTipTextStyle : _props$yTipTextStyle,
          _props$xTipBackground = props.xTipBackground,
          xTipBackground = _props$xTipBackground === void 0 ? defaultStyle.xTipBackground : _props$xTipBackground,
          _props$yTipBackground = props.yTipBackground,
          yTipBackground = _props$yTipBackground === void 0 ? defaultStyle.yTipBackground : _props$yTipBackground,
          _props$custom = props.custom,
          custom = _props$custom === void 0 ? false : _props$custom;

      var itemMarkerStyle = _objectSpread(_objectSpread({}, customItemMarkerStyle), defaultStyle.itemMarkerStyle);

      var xTipText = xScale.getText(records[0].origin[xField]);
      var yTipText = yScale.getText(records[0].origin[yField]);
      return jsx("group", null, snap ? records.map(function (item) {
        var x = item.x,
            y = item.y,
            color = item.color,
            shape = item.shape;
        return jsx("circle", {
          attrs: _objectSpread(_objectSpread({
            x: x,
            y: y,
            r: 3,
            stroke: color
          }, shape), tooltipMarkerStyle)
        });
      }) : null, jsx("group", {
        ref: this.rootRef,
        style: {
          left: coordLeft,
          top: layoutTop
        }
      }, !custom && jsx("group", {
        ref: this.itemsRef,
        style: _objectSpread(_objectSpread({}, defaultStyle.background), background),
        attrs: _objectSpread(_objectSpread({}, defaultStyle.background), background)
      }, showTitle ? jsx("text", {
        style: {
          marginBottom: '6px'
        },
        attrs: _objectSpread({
          text: firstOrigin[xField],
          fontSize: '24px',
          fill: '#fff',
          textAlign: 'start'
        }, titleStyle)
      }) : null, jsx("group", {
        style: {
          display: 'flex',
          flexDirection: 'row',
          flexWrap: 'wrap'
        }
      }, records.map(function (record) {
        var yValue = yScale.getText(record[yField]);
        var xValue = xScale.getText(record[xField]);
        return jsx("group", {
          style: {
            display: 'flex',
            flexDirection: 'row',
            alignItems: 'center',
            padding: [0, '10px', 0, 0]
          }
        }, showItemMarker ? jsx("marker", {
          style: {
            width: itemMarkerStyle.width,
            marginRight: '6px'
          },
          attrs: _objectSpread(_objectSpread({}, itemMarkerStyle), {}, {
            fill: record.color
          })
        }) : null, jsx("text", {
          attrs: _objectSpread(_objectSpread(_objectSpread({}, defaultStyle.nameStyle), nameStyle), {}, {
            text: yValue ? "".concat(xValue).concat(joinString) : xValue
          })
        }), jsx("text", {
          attrs: _objectSpread(_objectSpread(_objectSpread({}, defaultStyle.valueStyle), valueStyle), {}, {
            text: yValue
          })
        }));
      }))), showCrosshairs ? jsx("group", null, directionEnabled(crosshairsType, 'x') ? jsx("line", {
        attrs: _objectSpread(_objectSpread({
          x1: coordLeft,
          y1: y,
          x2: coordRight,
          y2: y
        }, defaultStyle.crosshairsStyle), crosshairsStyle)
      }) : null, directionEnabled(crosshairsType, 'y') ? jsx("line", {
        attrs: _objectSpread(_objectSpread({
          x1: x,
          y1: coordTop,
          x2: x,
          y2: coordBottom
        }, defaultStyle.crosshairsStyle), crosshairsStyle)
      }) : null) : null), showXTip && jsx("group", {
        style: _objectSpread(_objectSpread({
          left: x,
          top: coordBottom
        }, defaultStyle.xTipBackground), xTipBackground),
        attrs: _objectSpread(_objectSpread({}, defaultStyle.xTipBackground), xTipBackground)
      }, jsx("text", {
        attrs: _objectSpread(_objectSpread(_objectSpread({}, defaultStyle.xTipTextStyle), xTipTextStyle), {}, {
          text: isFunction(xTip) ? xTip(xTipText) : xTipText
        })
      })), showYTip && jsx("group", {
        style: _objectSpread(_objectSpread({
          left: coordLeft,
          top: y
        }, defaultStyle.yTipBackground), yTipBackground),
        attrs: _objectSpread(_objectSpread({}, defaultStyle.yTipBackground), yTipBackground)
      }, jsx("text", {
        attrs: _objectSpread(_objectSpread(_objectSpread({}, defaultStyle.yTipTextStyle), yTipTextStyle), {}, {
          text: isFunction(yTip) ? yTip(yTipText) : yTipText
        })
      })));
    }
  }]);

  return TooltipView;
}(Component);

export { TooltipView as default };