"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _createSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/createSuper"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _jsx = require("../../jsx");

var _component = _interopRequireDefault(require("../../base/component"));

var _util = require("@antv/util");

var _util2 = require("../../util");

var DEFAULT_CONFIG = {
  anchorOffset: 5,
  inflectionOffset: 15,
  sidePadding: 20,
  lineHeight: 32,
  adjustOffset: 15,
  skipOverlapLabels: false,
  triggerOn: 'touchstart',
  activeShape: false,
  activeStyle: {
    offset: 1,
    appendRadius: 8,
    fillOpacity: 0.5
  },
  label1OffsetY: -1,
  label2OffsetY: 1
};

function getEndPoint(center, angle, r) {
  return {
    x: center.x + r * Math.cos(angle),
    y: center.y + r * Math.sin(angle)
  };
} // 计算中间角度


function getMiddleAngle(startAngle, endAngle) {
  if (endAngle < startAngle) {
    endAngle += Math.PI * 2;
  }

  return (endAngle + startAngle) / 2;
} // 判断两个矩形是否相交
// function isOverlap(label1, label2) {
//   const label1BBox = label1.getBBox();
//   const label2BBox = label2.getBBox();
//   return (
//     Math.max(label1BBox.minX, label2BBox.minX) <=
//       Math.min(label1BBox.maxX, label2BBox.maxX) &&
//     Math.max(label1BBox.minY, label2BBox.minY) <=
//       Math.min(label1BBox.maxY, label2BBox.maxY)
//   );
// }


function findShapesByClass(shape, targetClassName) {
  var _ref = shape || {},
      _ref$_attrs = _ref._attrs,
      _attrs = _ref$_attrs === void 0 ? {} : _ref$_attrs;

  var children = _attrs.children,
      className = _attrs.className;
  var result = [];

  if (className === targetClassName) {
    result.push(shape);
  }

  if (children && children.length) {
    for (var i = 0, len = children.length; i < len; i++) {
      result.push.apply(result, (0, _toConsumableArray2.default)(findShapesByClass(children[i], targetClassName)));
    }
  }

  return result;
}

function findShapeByClassName(shape, point, className) {
  var targetShapes = findShapesByClass(shape, className);

  for (var i = 0, len = targetShapes.length; i < len; i++) {
    var _shape = targetShapes[i];

    if ((0, _util2.isInBBox)(_shape.getBBox(), point)) {
      return _shape;
    }
  }
}

var _default = function _default(View) {
  return /*#__PURE__*/function (_Component) {
    (0, _inherits2.default)(PieLabel, _Component);

    var _super = (0, _createSuper2.default)(PieLabel);

    function PieLabel(props) {
      var _this;

      (0, _classCallCheck2.default)(this, PieLabel);
      _this = _super.call(this, props);

      _this._handleEvent = function (ev) {
        var _this$props = _this.props,
            chart = _this$props.chart,
            onClick = _this$props.onClick;
        var ele = _this.triggerRef.current;
        var point = ev.points[0];
        var shape = findShapeByClassName(ele, point, 'click');
        var pieData = chart.getSnapRecords(point);

        if (typeof onClick === 'function') {
          // 点击label
          if (shape) {
            onClick(shape.get('data'));
          } // 点击饼图
          else if ((0, _util.isArray)(pieData) && pieData.length > 0) {
            onClick(pieData);
          }
        }
      };

      _this.props = (0, _util.deepMix)({}, DEFAULT_CONFIG, _this.props);
      _this.triggerRef = {};
      return _this;
    }

    (0, _createClass2.default)(PieLabel, [{
      key: "willMount",
      value: function willMount() {}
      /**
       * 绑定事件
       */

    }, {
      key: "didMount",
      value: function didMount() {
        this._initEvent();
      }
    }, {
      key: "getLabels",
      value: function getLabels() {
        var context = this.context;
        var _this$props2 = this.props,
            chart = _this$props2.chart,
            anchorOffset = _this$props2.anchorOffset,
            inflectionOffset = _this$props2.inflectionOffset,
            label1 = _this$props2.label1,
            label2 = _this$props2.label2,
            lineHeight = _this$props2.lineHeight;
        var coord = chart.coord;
        var center = coord.center,
            radius = coord.radius;
        var halves = [[], [] // right
        ]; // 存储左右 labels

        var labels = [];
        var geometry = chart.getGeometrys()[0];
        var records = geometry.records;
        records.forEach(function (record) {
          var children = record.children;
          var child = children[0];
          var xMin = child.xMin,
              xMax = child.xMax,
              color = child.color,
              origin = child.origin; // 算出锚点的基准位置

          var anchorAngle = getMiddleAngle(xMin, xMax);
          var anchorPoint = getEndPoint(center, anchorAngle, radius + anchorOffset);
          var inflectionPoint = getEndPoint(center, anchorAngle, radius + inflectionOffset);
          var label = {
            origin: origin,
            _anchor: anchorPoint,
            _inflection: inflectionPoint,
            x: inflectionPoint.x,
            y: inflectionPoint.y,
            r: radius + inflectionOffset,
            fill: color
          }; // 映射label1\label2

          if (typeof label1 === 'function') {
            label.label1 = label1(origin, color);
          }

          if (typeof label2 === 'function') {
            label.label2 = label2(origin, color);
          } // 判断文本的方向


          if (anchorPoint.x < center.x) {
            label._side = 'left';
            halves[0].push(label);
          } else {
            label._side = 'right';
            halves[1].push(label);
          }
        });
        var height = context.height; // @ts-ignore

        var maxCountForOneSide = (0, _util.toInteger)(height / lineHeight, 10);
        halves.forEach(function (half) {
          if (half.length > maxCountForOneSide) {
            half.splice(maxCountForOneSide, half.length - maxCountForOneSide);
          }

          half.sort(function (a, b) {
            return a.y - b.y;
          });
          labels = labels.concat(half);
        });
        return labels;
      }
    }, {
      key: "_initEvent",
      value: function _initEvent() {
        var context = this.context,
            props = this.props;
        var canvas = context.canvas;
        var triggerOn = props.triggerOn;
        canvas.on(triggerOn, this._handleEvent);
      }
    }, {
      key: "render",
      value: function render() {
        var labels = this.getLabels();
        return (0, _jsx.jsx)(View, (0, _objectSpread2.default)((0, _objectSpread2.default)({
          labels: labels
        }, this.props), {}, {
          triggerRef: this.triggerRef
        }));
      }
    }]);
    return PieLabel;
  }(_component.default);
};

exports.default = _default;