"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _get2 = _interopRequireDefault(require("@babel/runtime/helpers/get"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _createSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/createSuper"));

var _layout = _interopRequireDefault(require("../base/layout"));

var _util = require("@antv/util");

function transposedRect(_ref) {
  var xMin = _ref.xMin,
      xMax = _ref.xMax,
      yMin = _ref.yMin,
      yMax = _ref.yMax;
  return {
    xMin: yMin,
    xMax: yMax,
    yMin: xMin,
    yMax: xMax
  };
}

function convertRect(_ref2) {
  var x = _ref2.x,
      y = _ref2.y,
      size = _ref2.size,
      y0 = _ref2.y0;
  var xMin;
  var xMax;

  if ((0, _util.isArray)(x)) {
    xMin = x[0];
    xMax = x[1];
  } else {
    xMin = x - size / 2;
    xMax = x + size / 2;
  }

  var yMin;
  var yMax;

  if ((0, _util.isArray)(y)) {
    yMin = y[0];
    yMax = y[1];
  } else {
    yMin = Math.min(y0, y);
    yMax = Math.max(y0, y);
  }

  return {
    xMin: xMin,
    xMax: xMax,
    yMin: yMin,
    yMax: yMax
  };
}
/**
 * 直角坐标系
 * convert相关的方法，涉及将标准坐标系映射到实际坐标系内
 * transform相关的方法，是仅将某一种关键点转换成另一种关键点 (比如将x/y/size/y0转换成yMin/yMax/..)
 */


var Base = /*#__PURE__*/function (_Layout) {
  (0, _inherits2.default)(Base, _Layout);

  var _super = (0, _createSuper2.default)(Base);

  function Base(option) {
    var _this;

    (0, _classCallCheck2.default)(this, Base);
    _this = _super.call(this, option); // x y 调换

    _this.transposed = false; // x，y 的值域，在极坐标中对应的就是弧度和半径

    _this.x = [0, 1];
    _this.y = [0, 1];

    _this.update(option);

    return _this;
  }

  (0, _createClass2.default)(Base, [{
    key: "update",
    value: function update(option) {
      (0, _get2.default)((0, _getPrototypeOf2.default)(Base.prototype), "update", this).call(this, option);
      var left = this.left,
          top = this.top,
          width = this.width,
          height = this.height;
      this.center = {
        x: left + width / 2,
        y: top + height / 2
      };
      return this;
    } // 是循环， 比如极坐标是以 2π 循环的

  }, {
    key: "isCyclic",
    value: function isCyclic() {
      return false;
    } // 把归一后的值映射到对应的定义域

  }, {
    key: "convertPoint",
    value: function convertPoint(point) {
      return point;
    }
  }, {
    key: "convertRect",
    value: function convertRect(rectPoint) {
      var xRange = this.x,
          yRange = this.y;

      var _xRange = (0, _slicedToArray2.default)(xRange, 2),
          xStart = _xRange[0],
          xEnd = _xRange[1];

      var _yRange = (0, _slicedToArray2.default)(yRange, 2),
          yStart = _yRange[0],
          yEnd = _yRange[1];

      var _this$transformToRect = this.transformToRect(rectPoint),
          xMin = _this$transformToRect.xMin,
          xMax = _this$transformToRect.xMax,
          yMin = _this$transformToRect.yMin,
          yMax = _this$transformToRect.yMax;

      var x0 = xStart + (xEnd - xStart) * xMin;
      var x1 = xStart + (xEnd - xStart) * xMax;
      var y0 = yStart + (yEnd - yStart) * yMin;
      var y1 = yStart + (yEnd - yStart) * yMax;
      return {
        xMin: Math.min(x0, x1),
        xMax: Math.max(x0, x1),
        yMin: Math.min(y0, y1),
        yMax: Math.max(y0, y1)
      };
    }
  }, {
    key: "transformToRect",
    value: function transformToRect(rectPoint) {
      var transposed = this.transposed;
      var rect = convertRect(rectPoint);

      var _ref3 = transposed ? transposedRect(rect) : rect,
          xMin = _ref3.xMin,
          xMax = _ref3.xMax,
          yMin = _ref3.yMin,
          yMax = _ref3.yMax;

      return {
        xMin: xMin,
        xMax: xMax,
        yMin: yMin,
        yMax: yMax
      };
    } // 把canvas坐标的点位映射回归一后的值

  }, {
    key: "invertPoint",
    value: function invertPoint(point) {
      return point;
    }
  }]);
  return Base;
}(_layout.default);

var _default = Base;
exports.default = _default;